/* Name: accum_charge_control.c
 * Project: Sun_chargeV2.1
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2014-20-01
 * Tabsize: 4
 * Copyright: (c) 2014 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include <inttypes.h>
#include <avr/io.h>
#include <compat/deprecated.h>

#include "I_O_hardware.h"
#include "accum_charge_control.h"
#include "battery_calc.h"
#include "main.h"

/*
detection accum connection use signal from jump in connector (null voltage - SLA, Li accum, voltage bigger 2.5V - NiMH accum, voltage above null and below 2.5V - without accumulator,
 connector disconnected )
 
Check voltage level and turn off tracker in case full charge and mobile charger if accum is empty.

*/

int16_t accum_voltage_matrix [12];

void initArray ()
{

LI_ACCUM_EMPTY=K_INPUT_DEVIDER*LI_VOLTAGE_EMPTY*K_ADC;
LI_ACCUM_NORMAL=K_INPUT_DEVIDER*LI_VOLTAGE_NORMAL*K_ADC;
LI_ACCUM_FULL=K_INPUT_DEVIDER*LI_VOLTAGE_FULL*K_ADC;

SLA_6V_FULL=K_INPUT_DEVIDER*SLA_6V_VOLTAGE_FULL*K_ADC;
SLA_6V_NORMAL=K_INPUT_DEVIDER*SLA_6V_VOLTAGE_NORMAL*K_ADC;
SLA_6V_EMPTY=K_INPUT_DEVIDER*SLA_6V_VOLTAGE_EMPTY*K_ADC;
SLA_12V_FULL=K_INPUT_DEVIDER*SLA_12V_VOLTAGE_FULL*K_ADC;
SLA_12V_NORMAL=K_INPUT_DEVIDER*SLA_12V_VOLTAGE_NORMAL*K_ADC;
SLA_12V_EMPTY=K_INPUT_DEVIDER*SLA_12V_VOLTAGE_EMPTY*K_ADC;
NICD_FULL=K_INPUT_DEVIDER*NICD_VOLTAGE_FULL*K_ADC;
NICD_NORMAL=K_INPUT_DEVIDER*NICD_VOLTAGE_NORMAL*K_ADC;
NICD_EMPTY=K_INPUT_DEVIDER*NICD_VOLTAGE_EMPTY*K_ADC;

}




void indicatorChargeLevel (int16_t aAccumVoltage,int16_t emptyVoltage,int16_t normalVoltage, int16_t fullVoltage)
{

static uint8_t accumFullLevel;

 LED_FULL_OFF;
 LED_NORMAL_OFF;
 LED_EMPTY_OFF;
 
 
if (aAccumVoltage>normalVoltage) 
	{
	
	if (aAccumVoltage>fullVoltage) accumFullLevel=1;
	
	if (aAccumVoltage<(fullVoltage-fullVoltage/64)) accumFullLevel=0;
	
	
	if (accumFullLevel) LED_FULL_ON;
	else LED_NORMAL_ON;
	
	}
else 
	{
	
	if (aAccumVoltage<emptyVoltage) LED_EMPTY_ON;
	else
		{
		static uint16_t delay;
		
		if (delay<((aAccumVoltage-emptyVoltage)*64)) LED_NORMAL_ON;
		else LED_EMPTY_ON;
		
		if (delay<((normalVoltage-normalVoltage/64-emptyVoltage)*64)) delay++;
		else delay=0;
		
		}
		
		accumFullLevel=0;
	}


}

void indicatorVoltageLevel (int16_t aAccumVoltage,int16_t emptyVoltage, int16_t fullVoltage)
{
static uint16_t delay;
LED_FULL_OFF;
 LED_NORMAL_OFF;
 LED_EMPTY_OFF;
LI_LED_OFF;
SLA_6V_LED_OFF;
SLA_12V_LED_OFF;
NIMH_LED_OFF;

if (aAccumVoltage<emptyVoltage) LED_EMPTY_ON;
else
	{
	if (aAccumVoltage>fullVoltage) LED_FULL_ON;
	else LED_NORMAL_ON;
	}

if (delay) delay--;
else 
		delay=10000;
		
	if (delay<10)
	{
	LI_LED_ON;
	SLA_6V_LED_ON;
	SLA_12V_LED_ON;
	NIMH_LED_ON;
	}
	
}


uint8_t accumChargeControl(int16_t aAccumVoltage,int16_t aJumperVoltage,int16_t aAccumCurrent,int16_t aTemperature)
{
uint8_t mode=ESLALIACCUM;
static uint8_t NiMHcharged;
static uint8_t startZone;
int16_t correctTemperature;
uint8_t TrackerMode;
TrackerMode=WORK_TRACKER;
ChargeMode=CHARGE_ON;

if (aJumperVoltage>190) mode=EWITHOUTACCUM;
if (aJumperVoltage>600) mode=ENIMNHACCUM;

LI_LED_OFF;
SLA_6V_LED_OFF;
SLA_12V_LED_OFF;
NIMH_LED_OFF;



correctTemperature=aTemperature-134;




if (mode==ESLALIACCUM)
	{
	
	static int16_t zeroCurrentVoltage;
	{
	static uint16_t counterMeasurement=20000;
	static uint8_t  counterCurrentPause=40;
	
	
		
		if (counterMeasurement) counterMeasurement--;
				else
					{
					TrackerMode=STOP_TRACKER;
					ChargeMode=CHARGE_OFF;
					if (counterCurrentPause) counterCurrentPause--;
					else 
						{
						
						zeroCurrentVoltage=aAccumVoltage;
						counterMeasurement=20000;
						if (aAccumCurrent>2) counterCurrentPause=40;
						else counterCurrentPause=5;
						}
					
					}
				
	}


	
	
	if (aAccumVoltage<276) //LI accumulator
		{
		LI_LED_ON;
		indicatorChargeLevel(zeroCurrentVoltage,LI_ACCUM_EMPTY,LI_ACCUM_NORMAL,LI_ACCUM_FULL);
		if (zeroCurrentVoltage>LI_ACCUM_FULL) TrackerMode=STOP_TRACKER;

		}
	if (aAccumVoltage>275&&aAccumVoltage<580) //SLA 6V accumulator
		{
		SLA_6V_LED_ON;
		int16_t fullVoltage;
		fullVoltage=SLA_6V_FULL-correctTemperature/2-correctTemperature/8;
		indicatorChargeLevel(zeroCurrentVoltage,SLA_6V_EMPTY,SLA_6V_NORMAL,fullVoltage);
		if (zeroCurrentVoltage>fullVoltage) TrackerMode=STOP_TRACKER;
		}
	if (aAccumVoltage>579) //SLA 12V accumulator
		{
		SLA_12V_LED_ON;
		int16_t fullVoltage;
		fullVoltage=SLA_12V_FULL-correctTemperature+correctTemperature/8;
		indicatorChargeLevel(zeroCurrentVoltage,SLA_12V_EMPTY,SLA_12V_NORMAL,fullVoltage);
		if (zeroCurrentVoltage>fullVoltage) TrackerMode=STOP_TRACKER;
		}
	NiMHcharged=0;	
	}
if (mode==EWITHOUTACCUM)
	{
	
	indicatorVoltageLevel (aAccumVoltage,LI_VOLTAGE_EMPTY,ADC_VOLTAGE_LIMIT);
	
	if (aAccumVoltage>ADC_VOLTAGE_LIMIT) TrackerMode=STOP_TRACKER;
	NiMHcharged=0;
	}
if (mode==ENIMNHACCUM)
	{
	/*
	determination the end of charge:
	measure dV/dQ and charge stopped in situation
	  :
	 dV/dQ       3-  -      ,   (300mah)     ,
	 .
	      1   ,  dV     dQ,       
	  50,            .
	           1.   ( ) 1  
	*/
	NIMH_LED_ON;
	int16_t fullVoltage;
	static uint16_t counterMeasurement=20000;
	static uint8_t  counterCurrentPause=40,measureZeroCurrentVoltageFlag,filterChargeCounter;
	static int16_t chargeCounter,voltageArray[4],zeroCurrentVoltage,filterCurrent;
	
	if (aAccumCurrent>2)
		{
			{
			if (counterMeasurement) counterMeasurement--;
			else
				{
				TrackerMode=STOP_TRACKER;
				ChargeMode=CHARGE_OFF;
				measureZeroCurrentVoltageFlag=1;
				filterChargeCounter++;
				filterCurrent=filterCurrent+aAccumCurrent-filterCurrent/32;
				}
			}
			
			if (filterChargeCounter>9)	
			{
			chargeCounter=filterCurrent/32+chargeCounter;
			filterChargeCounter=0;
			}
		}
	else
		{
		counterMeasurement=60000;
		filterChargeCounter=0;
		chargeCounter=0;
		startZone=0;
		zeroCurrentVoltage=aAccumVoltage;
		voltageArray[0]=0;
		voltageArray[1]=0;
		voltageArray[2]=0;
		voltageArray[3]=0;
		}
	
	
	if (measureZeroCurrentVoltageFlag)	
		{
	
		TrackerMode=STOP_TRACKER;
		if (counterCurrentPause) counterCurrentPause--;
				else 
					{
					measureZeroCurrentVoltageFlag=0;
					zeroCurrentVoltage=aAccumVoltage;
					counterMeasurement=20000;
					counterCurrentPause=40;
			
					}
					
		}	
	
		
	if (chargeCounter>DELTA_AMPERHOUR) //calculation factory function 
		{
		
		int16_t derivative[4];
		
		
		derivative[0]=voltageArray[1]-voltageArray[0];
		derivative[1]=voltageArray[2]-voltageArray[1];
		derivative[2]=voltageArray[3]-voltageArray[2];
		derivative[3]=zeroCurrentVoltage-voltageArray[3];
		
		voltageArray[0]=voltageArray[1];
		voltageArray[1]=voltageArray[2];
		voltageArray[2]=voltageArray[3];
		voltageArray[3]=zeroCurrentVoltage;
		
		if (startZone==0)
			{
			if (derivative[3]>0&&derivative[2]>0&&derivative[1]>0&&derivative[0]>0) startZone=1;
			}
		if(startZone)
			{
			if (derivative[3]<1&&derivative[2]<1&&derivative[1]<1&&derivative[0]<1) NiMHcharged=1; //  
			
			if (derivative[3]>derivative[2]&&derivative[2]>derivative[1]&&derivative[1]>derivative[0]&&derivative[0]>0) NiMHcharged=1;//   
			}
		
		if (NiMHcharged) startZone=0;
		chargeCounter=0;
		}
		
		if (zeroCurrentVoltage>NICD_FULL) NiMHcharged=1;
		
		if (NiMHcharged) fullVoltage=0;
		else fullVoltage=NICD_FULL;
		
	indicatorChargeLevel(zeroCurrentVoltage,NICD_EMPTY,NICD_NORMAL,fullVoltage);
	
	
	if (NiMHcharged) TrackerMode=STOP_TRACKER;

	}
	
	

return TrackerMode;
}