
/* Name: main.c
 * Project: Sun_chargeV2.1
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2014-20-01
 * Tabsize: 4
 * Copyright: (c) 2014 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/*
General Description:
Sun_charge is embed program for ATmega88 controller that is part of Solar charger project (http://merk.wikidot.com/eng-electt-7w-solar-charger)
Project compiled by WinAVR-20090313 (AVR-GCC 4.3.2)
Sun_chargeV2.1 make functions control of level charge SLA accumulator with environment temperature correction, detect 6/12V level pluged SLA accumulator, 
detect Li accumulator, 4 NiMH accumulators, max power point tracker.
clk - Internal RC-oscillator 8 MHz, BOD disable.

*/
#include <stdlib.h>
#include <avr/wdt.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include <compat/deprecated.h>
#include<avr/sleep.h>

#include "main.h"
#include "accum_charge_control.h"
#include "phone_charger_control.h"
#include "power_supply_control.h"
#include "I_O_hardware.h"
#include "solar_MPPTracker.h"
#include "battery_calc.h"
#include "ADC.h"









//-------------------------------------------------------------------------------------------------------------------
void start_supply_voltage()
{

uint8_t o,p;
ACSR=_BV(ACD);

ADCSRA=_BV(ADEN)|_BV(ADSC)|_BV(ADPS2)|_BV(ADPS1);

ADMUX=0x46;

while (bit_is_set (ADCSRA,ADSC));
	o=ADCL;
	p=ADCH;
sbi (ADCSRA,ADSC);
while (bit_is_set (ADCSRA,ADSC));
	o=ADCL;
	p=ADCH;
if  (p*0x100+o<10)
	{
	TCCR0A=_BV(COM0A1)|_BV(WGM01)|_BV(WGM00);
	TCCR0B=_BV(CS00);
	sbi(DDRD,6);
	wdt_reset ();
	wdt_enable(WDTO_15MS);

	OCR0A=220;

	sbi (MCUCR,SE);
	sleep_mode ();

	}


}




//---------------------------------------------------------------------------------------------------------------------
int main (void)
{
struct inputADC copy,filteredValue;
start_supply_voltage();
initArray ();


ioinit ();


for (;;)
{		


if (MeasureFlag) //100us period
	{
	

	MeasureFlag=RESET;

		cli();	
	copy=data;
	sei();

	filter(&copy,&filteredValue);
	powerSupplyControl(copy.SourceVoltage);
	

	TrackerMode=accumChargeControl(filteredValue.AccumVoltage,copy.JumperVoltage,filteredValue.AccumCurrent,copy.Temperature);
	if (phoneChargeControl(copy.PhoneVoltageFeedback,copy.PhoneCurrentFeedback,filteredValue.AccumVoltage,filteredValue.PhoneCurrentFeedback,TrackerMode,copy.SourceVoltage,copy.JumperVoltage))
			solar_MPPTracker(TrackerMode,filteredValue.AccumVoltage);
	
	currentLevelIndicator(filteredValue.PhoneCurrentFeedback,filteredValue.AccumCurrent,copy.JumperVoltage);
	}
wdt_reset ();

sleep_mode ();
}

return(0);
}
