/* Name: phone_charger_control.c
 * Project: Sun_chargeV2.1
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2014-20-01
 * Tabsize: 4
 * Copyright: (c) 2014 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#include <inttypes.h>
#include <avr/io.h>
#include <compat/deprecated.h>

#include "main.h"
#include "I_O_hardware.h"
#include "phone_charger_control.h"




uint8_t phoneChargeControl(int16_t aPhoneVoltage,int16_t aPhoneCurrent,int16_t aInputVoltage,int16_t aPhoneFilteredCurrent,uint8_t trackerMode,int16_t aSourceVoltage,int16_t aJumperVoltage)
{
static int16_t integralBoost=10,lastCurrent,lastPWM1,lastPWM2,lastVoltage,Step=4;
int16_t error,PWM,chargeCurrent,chargeVoltage,deltaCurrent,deltaPWM,deltaVoltage;
static uint8_t mode;
static uint16_t delay;


if (SET_05A) chargeCurrent=CHARGE_CURRENT_04A;
else chargeCurrent=CHARGE_CURRENT_08A;



if (SET_52V) chargeVoltage=CHARGE_VOLTAGE_52V;
else chargeVoltage=CHARGE_VOLTAGE_56V;



if (aJumperVoltage>190&&aJumperVoltage<600)
	{
	if (aInputVoltage<BATTERY_DOWN_VOLTAGE_LIMIT||aSourceVoltage<45) 
		{
		OCR2B=0;
		OCR1B=10;
		integralBoost=120;
		}
	else
		{
		
		if (aPhoneVoltage>chargeVoltage) //phone Voltage < 5.5V
			{
		
			error=aPhoneCurrent-chargeCurrent;
		
			if (aInputVoltage>BATTERY_UPPER_VOLTAGE_LIMIT)
				{
				
				if((((deltaPWM>0&&deltaCurrent<0)||(deltaPWM<0&&deltaCurrent>0))&&(error>5))||error>10)
					{
					mode=MPPT;
					}
				if (error<-2) mode=REGULATOR;
			
				 OCR2B=(aInputVoltage-BATTERY_UPPER_VOLTAGE_LIMIT)/16+(aInputVoltage-BATTERY_UPPER_VOLTAGE_LIMIT)/32+12;
			
				sbi(TCCR2A,COM2B1);
				
				if (mode==MPPT) //power don't enough for set current
					{
					
					if (delay) delay--;
					else
						{
					
						deltaCurrent=lastCurrent-aPhoneFilteredCurrent;
						lastCurrent=aPhoneCurrent;
						deltaPWM=integralBoost-lastPWM1;
						
						lastPWM1=integralBoost;
						
						

						if (((deltaPWM>0)&&(deltaCurrent>0))||((deltaPWM<0)&&(deltaCurrent<0)))
							{
							Step=8;
							}
						if (((deltaPWM>0)&&(deltaCurrent<0))||((deltaPWM<0)&&(deltaCurrent>0)))
							{
							Step=-8;
							}
						integralBoost=integralBoost+Step;	
							
					
						delay=500;	
						}	
					if (integralBoost>320)
							{
							integralBoost=320;
							Step=-4;
							}
					if (integralBoost<8)
							{
								integralBoost=8;
								Step=4;
							}
						PWM=integralBoost/4;		
						
			
					}
				else
					{
					static uint16_t delay;
									
					if (delay) delay--;
					else						
						{							
					integralBoost=integralBoost+error;
					
					if (integralBoost>320)
							integralBoost=320;
					if (integralBoost<0)
								integralBoost=0;
					
						delay=800;
						}
						
						PWM=error/2+integralBoost/4;
				
							
					}
					if (PWM>90) PWM=90;
						if (PWM<2) PWM=2;
						if (aInputVoltage>BATTERY_LIMIT_VOLTAGE)
							{
							PWM=10;
							integralBoost=40;
							}
						
							if (trackerMode==STOP_TRACKER)	 OCR1B=0;
							else OCR1B=PWM;
							
				
					
				}
				else
				{
				OCR2B=0;
				OCR1B=20;
				integralBoost=80;
				mode=MPPT;
				}
			}	
		else
			{
			OCR1B=5;
			OCR2B=0;
			integralBoost--;
			cbi(TCCR2A,COM2B1);
			cbi(PORTD,3);
			}
		}
	
	return 0;
	}
else
	{
	if (ChargeMode==CHARGE_OFF||aInputVoltage<BATTERY_DOWN_VOLTAGE_LIMIT||aSourceVoltage<45) OCR2B=0; //
	else
		{

		if (aPhoneVoltage>chargeVoltage) //phone Voltage < 5.5V
			{
			
			sbi(TCCR2A,COM2B1);
								
								{
								static uint16_t delay;
								
								error=aPhoneCurrent-chargeCurrent;
									if (delay) delay--;
									else
									{
									integralBoost=integralBoost+error;
							
												
									

										if (integralBoost>340)
											integralBoost=340;
										if (integralBoost<0)
												integralBoost=0;
									delay=600;
							
									}
									PWM=error/2+integralBoost/4;
									
									if (PWM>90) PWM=90;
									if (PWM<0) PWM=0;
									
									 OCR2B=PWM;
							
									
								}
			}
		else 
			{
			OCR2B=0;
			
		cbi(TCCR2A,COM2B1);
			

		integralBoost--;

			cbi(PORTD,3);
	
			}
		}
	return 1;
	}
}