/* Name: phone_charger_control.h
 * Project: Sun_chargeV2.1
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2014-20-01
 * Tabsize: 4
 * Copyright: (c) 2014 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#ifndef __PHONE_CHARGE_CONTROL_H__
#define __PHONE_CHARGE_CONTROL_H__

#include "battery_calc.h"



uint8_t phoneChargeControl(int16_t aPhoneVoltage,int16_t aPhoneCurrent,int16_t aInputVoltage,int16_t aPhoneFilteredCurrent,uint8_t trackerMode,int16_t SourceVoltage,int16_t aJumperVoltage);


enum {REGULATOR=0,MPPT=1};



#define R2	10.0
#define R11 1.0
 
#define CURRENT_02A	0.2
#define CURRENT_027A 0.27
#define CURRENT_04A	0.4
#define CURRENT_05A	0.5
#define CURRENT_08A	0.8

#define Rshunt	0.22

#define CHARGE_CURRENT_02A (((AVR_POWER_VOLTAGE+Rshunt*CURRENT_02A)*R11/(R2+R11)-Rshunt*CURRENT_02A)*K_ADC)
#define CHARGE_CURRENT_027A (((AVR_POWER_VOLTAGE+Rshunt*CURRENT_027A)*R11/(R2+R11)-Rshunt*CURRENT_027A)*K_ADC)
#define CHARGE_CURRENT_04A (((AVR_POWER_VOLTAGE+Rshunt*CURRENT_04A)*R11/(R2+R11)-Rshunt*CURRENT_04A)*K_ADC)
#define CHARGE_CURRENT_05A (((AVR_POWER_VOLTAGE+Rshunt*CURRENT_05A)*R11/(R2+R11)-Rshunt*CURRENT_05A)*K_ADC)
#define CHARGE_CURRENT_08A (((AVR_POWER_VOLTAGE+Rshunt*CURRENT_08A)*R11/(R2+R11)-Rshunt*CURRENT_08A)*K_ADC)



#define PHONE_ZERO_CURRENT_POINT	((AVR_POWER_VOLTAGE*R11/(R2+R11))*K_ADC)


#define R36	12.0
#define R35	10.0

#define VOLTAGE_52V	5.2
#define VOLTAGE_56V	5.6



#define CHARGE_VOLTAGE_52V	((AVR_POWER_VOLTAGE-(AVR_POWER_VOLTAGE+VOLTAGE_52V)*R35/(R35+R36))*K_ADC)
#define CHARGE_VOLTAGE_56V	((AVR_POWER_VOLTAGE-(AVR_POWER_VOLTAGE+VOLTAGE_56V)*R35/(R35+R36))*K_ADC)

#define DOWN_VOLTAGE 3.0

#define BATTERY_DOWN_VOLTAGE_LIMIT DOWN_VOLTAGE*K_INPUT_DEVIDER*K_ADC

#define UPPER_VOLTAGE 5.0

#define BATTERY_UPPER_VOLTAGE_LIMIT UPPER_VOLTAGE*K_INPUT_DEVIDER*K_ADC

#define UP_LIMIT_VOLTAGE 15.0

#define BATTERY_LIMIT_VOLTAGE UP_LIMIT_VOLTAGE*K_INPUT_DEVIDER*K_ADC

#endif